/**
*
* Tsiolkovsky v1.0
* (c) 2005 Erik Anderson (Sputnik)
* erik_h_anderson@hotmail.com
*
*
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
* 
*
* Requires CVEL library --                             
* please see ProjectGanymede on SourceForge for the latest source *
*/

#include "Tsiolkovsky.h"
#include "cvel.h"

 
// ==============================================================
// Airfoil coefficient functions
// Return lift, moment and zero-lift drag coefficients as a
// function of angle of attack (alpha or beta)
// ==============================================================


ORION::ORION (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{
	hTsiol = oapiLoadMeshGlobal( "Tsiolkovsky");
	hTsiolFeet = oapiLoadMeshGlobal( "Tsiolfeet");
	hRock = oapiLoadMeshGlobal( "MDRock");
	tex   = oapiRegisterExhaustTexture ("Exhaust");

	// if we have orbitersound we load up some custom sounds

	MyID=ConnectToOrbiterSoundDLL3(GetHandle());
	RequestLoadVesselWave3(MyID,5,"Sound\\Mass-Driver\\MD.wav",BOTHVIEW_FADED_FAR);
/*
	if(!ConnectToOrbiterSoundDLL())
		havesound=false;
	else
	{
		havesound=true;
		RequestLoadVesselWave(5,"Sound\\Mass-Driver\\MD.wav");
	}
*/
	ph_main=NULL;
	Oberth = false;
	fuelmass = FuelMass;

	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if( strstr( classname, "OBERTH" ) ) {
		Oberth = true;
		fuelmass = FuelMassOberth;
	}


	//	AutoCalcStageMassChange(true);
}

void ORION::VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ) 
{
//	setstatus(status);
//	SetUpOrion();
	ClearMeshesEx();
	PlaceComponents();
    Draw(_V(0,0,0));
}



void ORION::TimeStep(  double simt )
{


	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
	}

	da = oapiGetSimStep() * SLAM_SPEED;
	SetThrusterGroupLevel(thg_orion, 0.0);
	SetAnimation (anim_slam, slam_proc);

	if ((DisplayTime > 0.0) && (oapiGetFocusObject() == GetHandle())) DisplayTime += oapiGetSimStep();
	if ((DisplayTime > 5.0) && (oapiGetFocusObject() == GetHandle())) {
		DisplayTime = 0.0;
		sprintf(oapiDebugString(),"");
	}
	
	GetGlobalPos(GloPos);
	Global2Local(_V(-GloPos.x,-GloPos.y,-GloPos.z),LocPos);

	sunangle = atan(LocPos.x/LocPos.z);

	panels_proc = sunangle/(2*PI);

	if (panels_proc < 0.0)
		panels_proc += 1.0;
	SetAnimation (anim_panels, panels_proc);


// Engine is on when the main throttle is up, or to continue the explosion cycle
	if (((GetThrusterGroupLevel(thg_main)) > 0.05) || (slam_proc-da > 0.0)) {
		WaitTime += oapiGetSimStep();
		EngineDelay = (1/(GetThrusterGroupLevel(thg_main)))-1;

		solardist = sqrt(GloPos.x*GloPos.x+GloPos.y*GloPos.y+GloPos.z*GloPos.z);
		sunmult = (149597870691/solardist)*(149597870691/solardist);
		powermult = sunmult*(sqrt(fabs(sin(sunangle))))*cos(atan(LocPos.y/LocPos.x));
		if (powermult < GetThrusterGroupLevel(thg_main))
			EngineDelay = (1/(powermult))-1;
//		sprintf(oapiDebugString(),"%.3f %.3f %.3f %.3f %.3f", sunmult, sin(sunangle), cos(atan(LocPos.y/LocPos.x)),powermult, EngineDelay);
	}

	if (WaitTime > EngineDelay)
		SlamCycle(da);


	if (slam_proc-da<0.0)			// if we're NOT cycling, re-vector the thrust angle
		firstnukestep = true;
//		VectorThrust();

	// Check if we're docked.  If we are, check if we need to refuel!

	if (GetFuelMass() < fuelmass)
		if (DockingStatus(0) == 1)
		{
			OBJHANDLE dockedvessel;
			dockedvessel = GetDockStatus(dock0);
			VESSEL *remotevessel = oapiGetVesselInterface(dockedvessel);					
			if (remotevessel->GetMass() > 50000000.0)
			{
				if (GetFuelMass()+(10.0*oapiGetTimeAcceleration()) < fuelmass)
					SetFuelMass(GetFuelMass()+10.0*oapiGetTimeAcceleration());
//				SetFuelMass(fuelmass);
			}
		}

	if (Oberth) {
		VESSELSTATUS vs;
		GetStatus(vs);
		if (vs.status == 1)
			{
				if (GetFuelMass()+(10.0*oapiGetTimeAcceleration()) < fuelmass)
					SetFuelMass(GetFuelMass()+10.0*oapiGetTimeAcceleration());
//				SetFuelMass(fuelmass);
			}
	}


/*	  if(gpds_gui.OperationComplete() && gpds_gui_active )
		{
		    // set this flag false to make sure
		    // we dont execute this routine again until
		    // we have run the GUI
			gpds_gui_active=false;

			// clear for a redraw
			ClearMeshesEx();

			// read in the created GPDS file
			ReadGPDSFile( "gpdsiofile.dat");
			PlaceComponents();
			Draw();
		}
*/
}


void ORION::SlamCycle(double da)
{
		PARTICLESTREAMSPEC exhaust_main = {
			0, 200, 0.2, 500, 0, 12, 50, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
			PARTICLESTREAMSPEC::ATM_FLAT, 0.3, 0.3
		};
		if((firstnukestep) && (GetHandle() == oapiGetFocusObject())) {
			StopVesselWave3(MyID,5);
			PlayVesselWave3(MyID,5);								// Play the sound!  BIG boom!
//			cloud = AddExhaustStream (th_orion[0],ExplosionPosition, &exhaust_main);

		}
		if (firstnukestep) DestroyNeighbors();
		firstnukestep = false;

		slam_proc += da;
//		if (!MoveNegative) slam_proc = slam_proc + da*2;	// Plate moves faster compressing...
//			else slam_proc = slam_proc - da;				// ...than expanding


		if (slam_proc+da*2 > 1.0)
		{
			slam_proc=0.0;				// FORCE the cycle ended
			WaitTime=0.0;
		};
		SetThrusterGroupLevel(thg_orion, 1.0);				// Set Orion thrust "on"

// Vary the exhaust texture to make the flash
//		DelExhaust(nuke);
//		nuke = AddExhaust (th_orion[0], 1.0, FlashSize*slam_proc*slam_proc*slam_proc*slam_proc, tex);

//		if ((MoveNegative) || (slam_proc > da)) DelExhaustStream (cloud);		// Stop making smoke puffs once we're done booming

}

void ORION::VectorThrust()
{
// Vector the thrust (exploding bombs slightly off-center) 
// Only happens BETWEEN explosions, of course, never during them.
	
		if (oapiGetTimeAcceleration() > 10) return;

		tempP = GetManualControlLevel(THGROUP_ATT_PITCHDOWN, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_PITCHUP, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

		tempY = GetManualControlLevel(THGROUP_ATT_YAWLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
			GetManualControlLevel(THGROUP_ATT_YAWRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);
	 
		GetAngularVel(AngularVel);
		tempP+=(10*AngularVel.x);			// Bias the thrust vector AGAINST the angular velocity a bit
		tempY+=(-10*AngularVel.y);			// This will damp angular velocity, preventing loss of control

		SetThrusterDir(th_orion[0],_V(0.008*tempY,0.008*tempP,1));
		firstnukestep = true;
}

void ORION::PlaceComponents()
{

	VECTOR3 pos = _V( 0,0,0.0 );
	COMMON_MESH tmp;
	tmp.MeshHandle(hTsiol);			
	tmp.MeshPosition( pos);			
	AddMeshEx( tmp);	
	
	if (Oberth) {
		tmp.MeshHandle(hTsiolFeet);			
		tmp.MeshPosition( pos);			
		AddMeshEx( tmp);
	}

	tmp.MeshHandle(hRock);			
	tmp.MeshPosition( pos);			
	AddMeshEx( tmp);
}

void ORION::DestroyNeighbors()
{
//	return;
	Local2Global (ExplosionPosition, gorion);  // global position of the mass-driver pellet
//	for (i = 0; i < oapiGetVesselCount(); i++) {
	j = oapiGetVesselCount();
	for (i = 0; i < j; i++) {
		j = oapiGetVesselCount();			// Last-ditch check to prevent using oapiGetVesselByIndex
		if (i >= j) return;					// if some other vessel has changed the index on us
		hV = oapiGetVesselByIndex (i);
		if (hV == GetHandle()) continue; // we don't want to damage ourselves ...
		oapiGetGlobalPos (hV, &gpos);
		distance = dist(gpos,gorion);
		if (distance < DestructRange) { // in range
			Global2Local (gpos,lpos);
//			lpos.x=lpos.x/distance;
//			lpos.y=lpos.y/distance;
//			lpos.z=lpos.z-(ExplosionPosition.z/2);
//			lpos.z=lpos.z/distance;
			if ((lpos.z < 0.0) && (lpos.x*lpos.x+lpos.y*lpos.y < oapiGetSize(hV)*oapiGetSize(hV)))  // Straight back from the center
													// We won't hit anything protected by our pusher plate
			{
				if (hV == oapiGetFocusObject())			// If we're trying to destroy the focus object
					oapiSetFocusObject(GetHandle());	// Switch to the destroying object
					
				VESSEL *remotevessel = oapiGetVesselInterface(hV);					
				VESSELSTATUS hv;
				oapiGetObjectName(hV,destructname,256);	
/*				if (!strnicmp (remotevessel->GetClassName(), "Orion", 5) != 0)		// If the target is an Orion also,
				{				
					oapiGetGlobalPos (GetHandle(), &gpos);
					remotevessel->Global2Local (gpos,lpos);		// Check to see if it's protected
					lpos.x=lpos.x/distance;						// by its pusher plate
					lpos.y=lpos.y/distance;
					lpos.z=lpos.z/distance;
					if ((lpos.z < 0.0) && (lpos.x*lpos.x+lpos.y*lpos.y < lpos.z*lpos.z)) { // Small angle-off from forward axis	
						sprintf(oapiDebugString(),"%s protected by pusher plate",destructname);
						continue;
					}
				}
*/	
				
//				if ((remotevessel->GetFuelMass() > 0.0) && (distance > (DestructRange / 2))) {					
					remotevessel->SetFuelMass(remotevessel->GetFuelMass()*0.75);		// Now it's lost some fuel
					sprintf(oapiDebugString(),"Damaged %s with a mass-driver pellet",destructname);
					DisplayTime += oapiGetSimStep();
//				}				
/*				else {
					remotevessel->Undock(ALLDOCKS);		// Undock everything.  Doesn't work right otherwise
					remotevessel->SetFuelMass(0.0);		// Damage it too.  Wouldn't want to forget that
					remotevessel->GetStatus(hv);
//					oapiDeleteVessel(hV);				// This works, but screws up the index
					hv.status=1;						// Instead, we'll set status to "landed"
					remotevessel->DefSetState(&hv);		// That'll fix 'em!
					sprintf(oapiDebugString(),"Destroyed %s due to nuclear blast",destructname);
				}
*/
			}
		}

	}
}


void ORION::SetClassCaps(  FILEHANDLE cfg )
{
	ph_main=NULL;

	DisplayTime = 0.0;
	WaitTime = 0.0;
	EngineDelay = 0.0;
	slam_proc = 0.0;
	panels_proc = 0.0;
//	jiggle_proc = -1.0;
//	bombcounter = 0;
	numevas = 0;
	firstnukestep = true;

	dock0 = CreateDock (_V( 0.0, 0.0 , 80.0), _V( 0, 0, 1), _V(0,1, 0));
/*	dock1 = CreateDock (_V( 1.9, 0.0 , 27.2), _V( 1, 0, 0), _V(0,0,-1));
	dock2 = CreateDock (_V(-1.9, 0.0 , 27.2), _V(-1, 0, 0), _V(0,0,-1));
	dock3 = CreateDock (_V( 0.0, 1.9 , 21.75), _V(0,  1, 0), _V(0,0, 1));
	dock4 = CreateDock (_V( 0.0,-1.9 , 21.75), _V(0, -1, 0), _V(0,0, 1));
	dock5 = CreateDock (_V( 1.9, 0.0 , 19.7), _V( 1, 0, 0), _V(0,0, 1));
	dock6 = CreateDock (_V(-1.9, 0.0 , 19.7), _V(-1, 0, 0), _V(0,0, 1));
*/
	SetSize (80);
	SetTouchdownPoints (_V(0,8.0,82), _V(-6,-8.0,82), _V(6,-8.0,82));
	SetPMI (_V(500,500,500));
	SetCrossSections (_V(44,75,15));
	SetCW (0.13, 0.5, 1.4, 1.4);
	SetRotDrag (_V(1.5,1.0,1.0));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
	ph_main = CreatePropellantResource( fuelmass );
	SetDefaultPropellantResource( ph_main );
//	ph_RCS = CreatePropellantResource( RCSFuelMass );
	SetEmptyMass ( EmptyMass + CalculateMass () );
//	SetEmptyMass ( CalculateMass (EmptyMass));
	MoveNegative=true;
//	AddMesh( hOrion, &pos );

	ClearMeshesEx();
	PlaceComponents();
    Draw();
	SetCameraOffset(_V(0,0,28.0));



	SetOrionStage();
	EnableTransponder(true);
	UnDockArm = false;

}




void ORION::SaveState( FILEHANDLE scn )
{
	SaveDefaultState (scn);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */

	/* save payload lines */
	SaveDefaultStateEx(scn);

}

void ORION::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{

		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
		ParsePayloadString(line);


		ParseScenarioLineEx (line, vs);

	}

	SetEmptyMass ( EmptyMass + CalculateMass () );
//	SetEmptyMass(  CalculateMass( EmptyMass )  );	

}

void ORION::SetOrionStage( )
{



	PARTICLESTREAMSPEC contrail = 
	{
		0, 5.0, 5, 250, 0.3, 4.0, 16, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
			PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
			PARTICLESTREAMSPEC::ATM_FLAT, 0.3, 0.3
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 200, 0.5, 500, 0, 7, 50, 0.0, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
			PARTICLESTREAMSPEC::ATM_FLAT, 0.3, 0.3
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");

/*	CreateAirfoil (LIFT_VERTICAL,   _V(0,0,-0.5), VLiftCoeff, 20, 270, 2.266);
	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-4), HLiftCoeff, 20,  50, 1.5);

	CreateControlSurface (AIRCTRL_ELEVATOR, 5.0, 1.5, _V( 0, 0,  -15), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_RUDDER,   2.0, 1.5, _V( 0, 3,  -16), AIRCTRL_AXIS_YPOS);
	CreateControlSurface (AIRCTRL_AILERON,  3.0, 1.5, _V( 7,-0.5,-15), AIRCTRL_AXIS_XPOS);
	CreateControlSurface (AIRCTRL_AILERON,  3.0, 1.5, _V(-7,-0.5,-15), AIRCTRL_AXIS_XNEG);

	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);
*/


	SetPanelsAnim();

	static UINT SlamGrp[1] = {0};
	static MGROUP_TRANSLATE Slam (1, SlamGrp, 1,
		_V(0.0,0.0,-1000));
	anim_slam = CreateAnimation (0.0);
	AddAnimationComponent (anim_slam, 0.0, 1.0, &Slam);
	SetAnimation (anim_slam, slam_proc);


	th_main[0] = CreateThruster (ExplosionPosition,    _V( 0.0,0.0,1.0), 
		1.0, ph_main, Ispmain, Ispmain);
	th_orion[0] = CreateThruster (ExplosionPosition,    _V( 0.0,0.0,1.0), 
		MainThrust, ph_main, Ispmain, Ispmain);

	thg_main  = CreateThrusterGroup( th_main, 1, THGROUP_MAIN );
	thg_orion = CreateThrusterGroup( th_orion, 1, THGROUP_USER );
//	cloud = AddExhaustStream (th_orion[0],ExplosionPosition, &exhaust_main);
	SoundOptionOnOff3(MyID,PLAYMAINTHRUST,FALSE);
//	SoundOptionOnOff(PLAYATTITUDETHRUST,FALSE);

//	nuke = AddExhaust (th_orion[0], 1.0, 100.0, tex);
	SetThrusterGroupLevel(thg_orion, 0.0);


	SetAttControlsOrion();
	EnableTransponder(true);
}



void ORION::SetPanelsAnim( )
{

	anim_panels = CreateAnimation (0.0);

	static UINT PanelGrp7[1] = {7};
	static MGROUP_ROTATE Panels7 (0,PanelGrp7, 1, 
		_V(0.0,0.0,-400.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels7);

	static UINT PanelGrp8[1] = {8};
	static MGROUP_ROTATE Panels8 (0,PanelGrp8, 1, 
		_V(0.0,0.0,-390.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels8);

	static UINT PanelGrp9[1] = {9};
	static MGROUP_ROTATE Panels9 (0,PanelGrp9, 1, 
		_V(0.0,0.0,-380.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels9);

	static UINT PanelGrp10[1] = {10};
	static MGROUP_ROTATE Panels10 (0,PanelGrp10, 1, 
		_V(0.0,0.0,-370.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels10);

	static UINT PanelGrp11[1] = {11};
	static MGROUP_ROTATE Panels11 (0,PanelGrp11, 1, 
		_V(0.0,0.0,-360.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels11);

	static UINT PanelGrp12[1] = {12};
	static MGROUP_ROTATE Panels12 (0,PanelGrp12, 1, 
		_V(0.0,0.0,-350.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels12);

	static UINT PanelGrp13[1] = {13};
	static MGROUP_ROTATE Panels13 (0,PanelGrp13, 1, 
		_V(0.0,0.0,-340.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels13);

	static UINT PanelGrp14[1] = {14};
	static MGROUP_ROTATE Panels14 (0,PanelGrp14, 1, 
		_V(0.0,0.0,-330.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels14);

	static UINT PanelGrp15[1] = {15};
	static MGROUP_ROTATE Panels15 (0,PanelGrp15, 1, 
		_V(0.0,0.0,-320.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels15);

	static UINT PanelGrp16[1] = {16};
	static MGROUP_ROTATE Panels16 (0,PanelGrp16, 1, 
		_V(0.0,0.0,-310.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels16);

	static UINT PanelGrp17[1] = {17};
	static MGROUP_ROTATE Panels17 (0,PanelGrp17, 1, 
		_V(0.0,0.0,-300.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels17);

	static UINT PanelGrp18[1] = {18};
	static MGROUP_ROTATE Panels18 (0,PanelGrp18, 1, 
		_V(0.0,0.0,-290.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels18);

	static UINT PanelGrp19[1] = {19};
	static MGROUP_ROTATE Panels19 (0,PanelGrp19, 1, 
		_V(0.0,0.0,-280.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels19);

	static UINT PanelGrp20[1] = {20};
	static MGROUP_ROTATE Panels20 (0,PanelGrp20, 1, 
		_V(0.0,0.0,-270.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels20);

	static UINT PanelGrp21[1] = {21};
	static MGROUP_ROTATE Panels21 (0,PanelGrp21, 1, 
		_V(0.0,0.0,-260.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels21);

	static UINT PanelGrp22[1] = {22};
	static MGROUP_ROTATE Panels22 (0,PanelGrp22, 1, 
		_V(0.0,0.0,-250.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels22);

	static UINT PanelGrp23[1] = {23};
	static MGROUP_ROTATE Panels23 (0,PanelGrp23, 1, 
		_V(0.0,0.0,-240.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels23);

	static UINT PanelGrp24[1] = {24};
	static MGROUP_ROTATE Panels24 (0,PanelGrp24, 1, 
		_V(0.0,0.0,-230.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels24);

	static UINT PanelGrp25[1] = {25};
	static MGROUP_ROTATE Panels25 (0,PanelGrp25, 1, 
		_V(0.0,0.0,-220.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels25);

	static UINT PanelGrp26[1] = {26};
	static MGROUP_ROTATE Panels26 (0,PanelGrp26, 1, 
		_V(0.0,0.0,-210.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels26);

	static UINT PanelGrp27[1] = {27};
	static MGROUP_ROTATE Panels27 (0,PanelGrp27, 1, 
		_V(0.0,0.0,-200.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels27);

	static UINT PanelGrp28[1] = {28};
	static MGROUP_ROTATE Panels28 (0,PanelGrp28, 1, 
		_V(0.0,0.0,-190.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels28);

	static UINT PanelGrp29[1] = {29};
	static MGROUP_ROTATE Panels29 (0,PanelGrp29, 1, 
		_V(0.0,0.0,-180.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels29);

	static UINT PanelGrp30[1] = {30};
	static MGROUP_ROTATE Panels30 (0,PanelGrp30, 1, 
		_V(0.0,0.0,-170.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels30);

	static UINT PanelGrp31[1] = {31};
	static MGROUP_ROTATE Panels31 (0,PanelGrp31, 1, 
		_V(0.0,0.0,-160.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels31);

	static UINT PanelGrp32[1] = {32};
	static MGROUP_ROTATE Panels32 (0,PanelGrp32, 1, 
		_V(0.0,0.0,-150.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels32);

	static UINT PanelGrp33[1] = {33};
	static MGROUP_ROTATE Panels33 (0,PanelGrp33, 1, 
		_V(0.0,0.0,-140.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels33);

	static UINT PanelGrp34[1] = {34};
	static MGROUP_ROTATE Panels34 (0,PanelGrp34, 1, 
		_V(0.0,0.0,-130.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels34);

	static UINT PanelGrp35[1] = {35};
	static MGROUP_ROTATE Panels35 (0,PanelGrp35, 1, 
		_V(0.0,0.0,-120.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels35);

	static UINT PanelGrp36[1] = {36};
	static MGROUP_ROTATE Panels36 (0,PanelGrp36, 1, 
		_V(0.0,0.0,-110.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels36);

	static UINT PanelGrp37[1] = {37};
	static MGROUP_ROTATE Panels37 (0,PanelGrp37, 1, 
		_V(0.0,0.0,-100.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels37);

	static UINT PanelGrp38[1] = {38};
	static MGROUP_ROTATE Panels38 (0,PanelGrp38, 1, 
		_V(0.0,0.0,-90.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels38);

	static UINT PanelGrp39[1] = {39};
	static MGROUP_ROTATE Panels39 (0,PanelGrp39, 1, 
		_V(0.0,0.0,-80.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels39);

	static UINT PanelGrp40[1] = {40};
	static MGROUP_ROTATE Panels40 (0,PanelGrp40, 1, 
		_V(0.0,0.0,-70.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels40);

	static UINT PanelGrp41[1] = {41};
	static MGROUP_ROTATE Panels41 (0,PanelGrp41, 1, 
		_V(0.0,0.0,-60.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels41);

	static UINT PanelGrp42[1] = {42};
	static MGROUP_ROTATE Panels42 (0,PanelGrp42, 1, 
		_V(0.0,0.0,-50.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels42);

	static UINT PanelGrp43[1] = {43};
	static MGROUP_ROTATE Panels43 (0,PanelGrp43, 1, 
		_V(0.0,0.0,-40.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels43);

	static UINT PanelGrp44[1] = {44};
	static MGROUP_ROTATE Panels44 (0,PanelGrp44, 1, 
		_V(0.0,0.0,-30.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels44);

	static UINT PanelGrp45[1] = {45};
	static MGROUP_ROTATE Panels45 (0,PanelGrp45, 1, 
		_V(0.0,0.0,-20.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels45);

	static UINT PanelGrp46[1] = {46};
	static MGROUP_ROTATE Panels46 (0,PanelGrp46, 1, 
		_V(0.0,0.0,-10.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels46);


	static UINT PanelGrp47[1] = {47};
	static MGROUP_ROTATE Panels47 (0,PanelGrp47, 1, 
		_V(0.0,0.0,-410.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels47);

	static UINT PanelGrp48[1] = {48};
	static MGROUP_ROTATE Panels48 (0,PanelGrp48, 1, 
		_V(0.0,0.0,-420.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels48);

	static UINT PanelGrp49[1] = {49};
	static MGROUP_ROTATE Panels49 (0,PanelGrp49, 1, 
		_V(0.0,0.0,-430.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels49);

	static UINT PanelGrp50[1] = {50};
	static MGROUP_ROTATE Panels50 (0,PanelGrp50, 1, 
		_V(0.0,0.0,-440.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels50);

	static UINT PanelGrp51[1] = {51};
	static MGROUP_ROTATE Panels51 (0,PanelGrp51, 1, 
		_V(0.0,0.0,-450.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels51);

	static UINT PanelGrp52[1] = {52};
	static MGROUP_ROTATE Panels52 (0,PanelGrp52, 1, 
		_V(0.0,0.0,-460.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels52);

	static UINT PanelGrp53[1] = {53};
	static MGROUP_ROTATE Panels53 (0,PanelGrp53, 1, 
		_V(0.0,0.0,-470.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels53);

	static UINT PanelGrp54[1] = {54};
	static MGROUP_ROTATE Panels54 (0,PanelGrp54, 1, 
		_V(0.0,0.0,-480.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels54);

	static UINT PanelGrp55[1] = {55};
	static MGROUP_ROTATE Panels55 (0,PanelGrp55, 1, 
		_V(0.0,0.0,-490.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels55);

	static UINT PanelGrp56[1] = {56};
	static MGROUP_ROTATE Panels56 (0,PanelGrp56, 1, 
		_V(0.0,0.0,-500.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels56);

	static UINT PanelGrp57[1] = {57};
	static MGROUP_ROTATE Panels57 (0,PanelGrp57, 1, 
		_V(0.0,0.0,-510.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels57);

	static UINT PanelGrp58[1] = {58};
	static MGROUP_ROTATE Panels58 (0,PanelGrp58, 1, 
		_V(0.0,0.0,-520.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels58);

	static UINT PanelGrp59[1] = {59};
	static MGROUP_ROTATE Panels59 (0,PanelGrp59, 1, 
		_V(0.0,0.0,-530.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels59);

	static UINT PanelGrp60[1] = {60};
	static MGROUP_ROTATE Panels60 (0,PanelGrp60, 1, 
		_V(0.0,0.0,-540.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels60);

	static UINT PanelGrp61[1] = {61};
	static MGROUP_ROTATE Panels61 (0,PanelGrp61, 1, 
		_V(0.0,0.0,-550.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels61);

	static UINT PanelGrp62[1] = {62};
	static MGROUP_ROTATE Panels62 (0,PanelGrp62, 1, 
		_V(0.0,0.0,-560.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels62);

	static UINT PanelGrp63[1] = {63};
	static MGROUP_ROTATE Panels63 (0,PanelGrp63, 1, 
		_V(0.0,0.0,-570.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels63);

	static UINT PanelGrp64[1] = {64};
	static MGROUP_ROTATE Panels64 (0,PanelGrp64, 1, 
		_V(0.0,0.0,-580.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels64);

	static UINT PanelGrp65[1] = {65};
	static MGROUP_ROTATE Panels65 (0,PanelGrp65, 1, 
		_V(0.0,0.0,-590.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels65);

	static UINT PanelGrp66[1] = {66};
	static MGROUP_ROTATE Panels66 (0,PanelGrp66, 1, 
		_V(0.0,0.0,-600.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels66);

	static UINT PanelGrp67[1] = {67};
	static MGROUP_ROTATE Panels67 (0,PanelGrp67, 1, 
		_V(0.0,0.0,-610.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels67);

	static UINT PanelGrp68[1] = {68};
	static MGROUP_ROTATE Panels68 (0,PanelGrp68, 1, 
		_V(0.0,0.0,-620.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels68);

	static UINT PanelGrp69[1] = {69};
	static MGROUP_ROTATE Panels69 (0,PanelGrp69, 1, 
		_V(0.0,0.0,-630.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels69);

	static UINT PanelGrp70[1] = {70};
	static MGROUP_ROTATE Panels70 (0,PanelGrp70, 1, 
		_V(0.0,0.0,-640.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels70);

	static UINT PanelGrp71[1] = {71};
	static MGROUP_ROTATE Panels71 (0,PanelGrp71, 1, 
		_V(0.0,0.0,-650.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels71);

	static UINT PanelGrp72[1] = {72};
	static MGROUP_ROTATE Panels72 (0,PanelGrp72, 1, 
		_V(0.0,0.0,-660.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels72);

	static UINT PanelGrp73[1] = {73};
	static MGROUP_ROTATE Panels73 (0,PanelGrp73, 1, 
		_V(0.0,0.0,-670.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels73);

	static UINT PanelGrp74[1] = {74};
	static MGROUP_ROTATE Panels74 (0,PanelGrp74, 1, 
		_V(0.0,0.0,-680.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels74);

	static UINT PanelGrp75[1] = {75};
	static MGROUP_ROTATE Panels75 (0,PanelGrp75, 1, 
		_V(0.0,0.0,-690.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels75);

	static UINT PanelGrp76[1] = {76};
	static MGROUP_ROTATE Panels76 (0,PanelGrp76, 1, 
		_V(0.0,0.0,-700.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels76);

	static UINT PanelGrp77[1] = {77};
	static MGROUP_ROTATE Panels77 (0,PanelGrp77, 1, 
		_V(0.0,0.0,-710.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels77);

	static UINT PanelGrp78[1] = {78};
	static MGROUP_ROTATE Panels78 (0,PanelGrp78, 1, 
		_V(0.0,0.0,-720.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels78);

	static UINT PanelGrp79[1] = {79};
	static MGROUP_ROTATE Panels79 (0,PanelGrp79, 1, 
		_V(0.0,0.0,-730.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels79);

	static UINT PanelGrp80[1] = {80};
	static MGROUP_ROTATE Panels80 (0,PanelGrp80, 1, 
		_V(0.0,0.0,-740.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels80);

	static UINT PanelGrp81[1] = {81};
	static MGROUP_ROTATE Panels81 (0,PanelGrp81, 1, 
		_V(0.0,0.0,-750.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels81);

	static UINT PanelGrp82[1] = {82};
	static MGROUP_ROTATE Panels82 (0,PanelGrp82, 1, 
		_V(0.0,0.0,-760.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels82);

	static UINT PanelGrp83[1] = {83};
	static MGROUP_ROTATE Panels83 (0,PanelGrp83, 1, 
		_V(0.0,0.0,-770.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels83);

	static UINT PanelGrp84[1] = {84};
	static MGROUP_ROTATE Panels84 (0,PanelGrp84, 1, 
		_V(0.0,0.0,-780.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels84);

	static UINT PanelGrp85[1] = {85};
	static MGROUP_ROTATE Panels85 (0,PanelGrp85, 1, 
		_V(0.0,0.0,-790.0),_V(0,1,0),360*RAD);
	AddAnimationComponent (anim_panels, 0.0, 1.0, &Panels85);


	SetAnimation (anim_panels, panels_proc);

}


//URWUMPE:
/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */

void ORION::SeparateEVA ()
{
	VESSELSTATUS vs;
	GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vs.fuel=1.0;
	vs.status=0;
	VECTOR3 ofs = _V(0,0,80.0);
	Local2Rel (ofs, vs.rpos);
	char name[256], name2[256];
//	strcpy (name, GetName()); strcat (name, "-PLT");


	UINT i;
	for (i=1;i<50;i++) {
			strcpy (name2, GetName()); 
			sprintf(name, "-EV%d", i);
			strcat (name2, name);
			hvessel=oapiGetVesselByName(name2);
			if (hvessel == 0) break;
	}		
	if ( i < 49) 
	{
		VESSEL::Create (name2, "gemini_leva", vs);
		hvessel=oapiGetVesselByName(name2);
		oapiSetFocusObject(hvessel);
	}



//	VESSEL::Create (name, "gemini_leva", vs);
}




int ORION::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
		if (KEYDOWN (keystate, OAPI_KEY_J) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) ) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
			{

//
//  This part does the Z-axis rotation for the launched vessel
//

				VESSEL *depV;
				VECTOR3 glopos, locpos;
				VESSELSTATUS vs;
				double payrot;

				depV = DeployPayload(_V(0,0,0));
				if (depV || NULL) {
					depV->GetGlobalPos(glopos);
					Global2Local (glopos, locpos);
					depV->GetStatus(vs);
					if (strnicmp (depV->GetClassName(), "zrot", 4) == 0) {	// If the target class starts with "zrot" in it, 
																				//	then we can do this routine
/*						if ((!strnicmp(depV->GetName(), "#", 1) != 0) 		// If the name starts with "#" or "@#",
							|| (!strnicmp(depV->GetName(), "@#", 2) != 0)) {	// then we need to do a 180 Y-rotation
//								depV->GetStatus(vs);
//								Global2Local (vs.arot,locrot);
								locrot = _V(PI,0,0);
								GlobalRot (locrot,vs.arot);
//								depV->DefSetState(&vs);
						}
*/
//						depV->GetStatus(vs);
						payrot = atan(locpos.x/locpos.y);
						if ((locpos.y < 0)) payrot += PI;
						if (strnicmp (depV->GetClassName(), "zrot180", 7) != 0)		// If it starts with zrot180, spin it another 180
							payrot += PI;
//						if (strnicmp(depV->GetName(), "@", 1) == 0)							// or it starts with an "@"
//							payrot += PI;
						vs.arot.z += payrot;
						depV->DefSetState(&vs);
					}
				}

				ClearMeshesEx();
				PlaceComponents();
				Draw(_V(0,0,0));
				SetEmptyMass ( EmptyMass + CalculateMass () );
				SetAttControlsOrion();
			}
			UnDockArm = false;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_C) && KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) ) 
		{ // "Jettison Stage"
//			if (oapiAcceptDelayedKey (OAPI_KEY_C, 1.0))
//				dock1=CreateDock( _V(0,0.0,2.0), _V(0,0,1), _V(0,1,0));
			
			UnDockArm = false;
			return 1;
		}
		
/*		if (KEYDOWN (keystate, OAPI_KEY_N) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) ) 
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_N, 1.0))
			{

				try{
					if(gpds_gui.OperationComplete() )
					{
						StartGPDSGUIEx("GPDSRocketBuilder2.exe");
						gpds_gui_active=true;

					}
				}
				catch(...)
				{
				MessageBox( NULL,"Error","Couldnt Start GUI Companion",MB_ICONINFORMATION);

				}
			}
			
			UnDockArm = false;
			return 1;
		}
*/		
		if (KEYDOWN (keystate, OAPI_KEY_E) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) ) 
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
		{
			
			
				SeparateEVA();
				SetOrionStage();
/*
				VESSELSTATUS vs;
				OBJHANDLE hvessel;
				GetStatus (vs);
				vs.eng_main = vs.eng_hovr = 0.0;
				vs.fuel=1.0;
				vs.status=0;
				VECTOR3 ofs = _V(1.8,1.8,24.25);
				Local2Rel (ofs, vs.rpos);
				char name[256], name2[256];
				numevas++;
				strcpy (name2, GetName()); 
				sprintf(name, " EVA-%d", numevas);
				strcat (name2, name);
	
				VECTOR3 vel = _V(0,0.0,0.0);
				VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				if (numevas < 9) {
					VESSEL::Create (name2, "Gemini_leva", vs);
					hvessel=oapiGetVesselByName(name2);
					oapiSetFocusObject(hvessel);
				}
*/	
			}
			
			UnDockArm = false;
			return 1;
		}
/*		if (KEYDOWN (keystate, OAPI_KEY_D) && KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) ) 
		{ // "Undock"
			if (!UnDockArm)
				UnDockArm = true;
			else {
				Undock(ALLDOCKS);
				UnDockArm = false;
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_0) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) && UnDockArm ) 
		{ // "Undock the numbered port"
			Undock(0);
			UnDockArm = false;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_1) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) && UnDockArm ) 
		{ // "Undock the numbered port"
			Undock(1);
			UnDockArm = false;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_2) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) && UnDockArm ) 
		{ // "Undock the numbered port"
			Undock(2);
			UnDockArm = false;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_3) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) && UnDockArm ) 
		{ // "Undock the numbered port"
			Undock(3);
			UnDockArm = false;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_4) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) && UnDockArm ) 
		{ // "Undock the numbered port"
			Undock(4);
			UnDockArm = false;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_5) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) && UnDockArm ) 
		{ // "Undock the numbered port"
			Undock(5);
			UnDockArm = false;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_6) && !KEYMOD_CONTROL(keystate) && !KEYMOD_SHIFT(keystate) && UnDockArm ) 
		{ // "Undock the numbered port"
			Undock(6);
			UnDockArm = false;
			return 1;
		}
*/
		
	return 0;
}







void ORION::SetAttControlsOrion()
{

//	VECTOR3 v_com;
//	double plmass;

//	plmass = CalculateMass(&v_com);

//	double comshift = -1.0*(v_com.z * plmass)/(GetMass());

	double comshift = 0.0;

	PARTICLESTREAMSPEC RPL_exhaust = {
		0, .65, 600, 25.0, 1.0, 0.7, -.5, 0.5, PARTICLESTREAMSPEC::EMISSIVE,
			PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
			PARTICLESTREAMSPEC::ATM_FLAT, 0.3, 0.3
	};

	THRUSTER_HANDLE th_att_rot[4], th_att_lin[4];
	th_att_rot[0] = CreateThruster (_V(0,-18, 24.0-comshift), _V(0, 1,0), AttThrust, ph_main, IspGYRO, IspRCS);
	th_att_rot[1] = CreateThruster (_V(0, 49,-24.0-comshift), _V(0,-1,0), AttThrust, ph_main, IspGYRO, IspRCS);
	th_att_rot[2] = CreateThruster (_V(0, 18, 24.0-comshift), _V(0,-1,0), AttThrust, ph_main, IspGYRO, IspRCS);
	th_att_rot[3] = CreateThruster (_V(0,-49,-24.0-comshift), _V(0, 1,0), AttThrust, ph_main, IspGYRO, IspRCS);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_PITCHUP);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_PITCHDOWN);
	AddExhaustStream(th_att_rot[1], _V( 11.2,0, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[0], _V( 35.0,0,48.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[1], _V(-11.2,0, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[0], _V(-35.0,0,48.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[3], _V( 11.2,0, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[2], _V( 35.0,0,48.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[3], _V(-11.2,0, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[2], _V(-35.0,0,48.0),&RPL_exhaust);

//	AddExhaust (th_att_rot[0], 1.5, 0.1, tex);
//	AddExhaust (th_att_rot[1], 1.5, 0.1, tex);
//	AddExhaust (th_att_rot[2], 1.5, 0.1, tex);
//	AddExhaust (th_att_rot[3], 1.5, 0.1, tex);

	th_att_lin[0] = CreateThruster (_V(0,-18, 24.0), _V(0, 1,0), AttThrust, ph_main, IspRCS, IspRCS);
	th_att_lin[3] = CreateThruster (_V(0, 49,-6.0), _V(0,-1,0), AttThrust*4, ph_main, IspRCS, IspRCS);
	th_att_lin[2] = CreateThruster (_V(0, 18, 24.0), _V(0,-1,0), AttThrust, ph_main, IspRCS, IspRCS);
	th_att_lin[1] = CreateThruster (_V(0,-49,-6.0), _V(0, 1,0), AttThrust*4, ph_main, IspRCS, IspRCS);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_UP);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_DOWN);
	AddExhaustStream(th_att_lin[1], _V( 11.2,0, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[0], _V( 35.0,0,48.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[1], _V(-11.2,0, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[0], _V(-35.0,0,48.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[3], _V( 11.2,0, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[2], _V( 35.0,0,48.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[3], _V(-11.2,0, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[2], _V(-35.0,0,48.0),&RPL_exhaust);
//	AddExhaust (th_att_lin[0], 1.5, 0.1, tex);
//	AddExhaust (th_att_lin[1], 1.5, 0.1, tex);
//	AddExhaust (th_att_lin[2], 1.5, 0.1, tex);
//	AddExhaust (th_att_lin[3], 1.5, 0.1, tex);

	th_att_rot[0] = CreateThruster (_V( 18,0, 24.0-comshift), _V(-1,0,0), AttThrust, ph_main, IspGYRO, IspRCS);
	th_att_rot[1] = CreateThruster (_V(-49,0,-24.0-comshift), _V( 1,0,0), AttThrust, ph_main, IspGYRO, IspRCS);
	th_att_rot[2] = CreateThruster (_V(-18,0, 24.0-comshift), _V( 1,0,0), AttThrust, ph_main, IspGYRO, IspRCS);
	th_att_rot[3] = CreateThruster (_V( 49,0,-24.0-comshift), _V(-1,0,0), AttThrust, ph_main, IspGYRO, IspRCS);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_YAWLEFT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_YAWRIGHT);
	AddExhaustStream(th_att_rot[1], _V(0, 11.2, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[1], _V(0,-11.2, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[3], _V(0, 11.2, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[3], _V(0,-11.2, 1.0),&RPL_exhaust);
//	AddExhaust (th_att_rot[0], 1.5, 0.1, tex);
//	AddExhaust (th_att_rot[1], 1.5, 0.1, tex);
//	AddExhaust (th_att_rot[2], 1.5, 0.1, tex);
//	AddExhaust (th_att_rot[3], 1.5, 0.1, tex);

	th_att_lin[0] = CreateThruster (_V( 19,0, 24.0), _V(-1,0,0), AttThrust, ph_main, IspRCS, IspRCS);
	th_att_lin[3] = CreateThruster (_V(-49,0,-6.0), _V( 1,0,0), AttThrust*4, ph_main, IspRCS, IspRCS);
	th_att_lin[2] = CreateThruster (_V(-19,0, 24.0), _V( 1,0,0), AttThrust, ph_main, IspRCS, IspRCS);
	th_att_lin[1] = CreateThruster (_V( 49,0,-6.0), _V(-1,0,0), AttThrust*4, ph_main, IspRCS, IspRCS);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_LEFT);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_RIGHT);
	AddExhaustStream(th_att_lin[0], _V(0, 11.2, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[1], _V(0,-11.2, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[2], _V(0, 11.2, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[3], _V(0,-11.2, 1.0),&RPL_exhaust);
//	AddExhaust (th_att_lin[0], 1.5, 0.1, tex);
//	AddExhaust (th_att_lin[1], 1.5, 0.1, tex);
//	AddExhaust (th_att_lin[2], 1.5, 0.1, tex);
//	AddExhaust (th_att_lin[3], 1.5, 0.1, tex);

	th_att_rot[0] = CreateThruster (_V( 48,0,0), _V(0, 1,0), AttThrust, ph_main, IspGYRO, IspRCS);
	th_att_rot[1] = CreateThruster (_V(-48,0,0), _V(0,-1,0), AttThrust, ph_main, IspGYRO, IspRCS);
	th_att_rot[2] = CreateThruster (_V(-48,0,0), _V(0, 1,0), AttThrust, ph_main, IspGYRO, IspRCS);
	th_att_rot[3] = CreateThruster (_V( 48,0,0), _V(0,-1,0), AttThrust, ph_main, IspGYRO, IspRCS);
	CreateThrusterGroup (th_att_rot,   2, THGROUP_ATT_BANKLEFT);
	CreateThrusterGroup (th_att_rot+2, 2, THGROUP_ATT_BANKRIGHT);
	AddExhaustStream(th_att_rot[0], _V( 11.2, 0, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[1], _V(-11.2, 0, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[3], _V( 11.2, 0, 1.0),&RPL_exhaust);
	AddExhaustStream(th_att_rot[2], _V(-11.2, 0, 1.0),&RPL_exhaust);
//	AddExhaust (th_att_rot[0], 1.5, 0.1, tex);
//	AddExhaust (th_att_rot[1], 1.5, 0.1, tex);
//	AddExhaust (th_att_rot[2], 1.5, 0.1, tex);
//	AddExhaust (th_att_rot[3], 1.5, 0.1, tex);

	th_att_lin[0] = CreateThruster (_V( 49,0, -6.0), _V(0,0, 1), AttThrust*4, ph_main, IspRCS, IspRCS);
	th_att_lin[1] = CreateThruster (_V(-49,0, -6.0), _V(0,0, 1), AttThrust*4, ph_main, IspRCS, IspRCS);
	th_att_lin[2] = CreateThruster (_V(-19,0, 24.0), _V(0,0,-1), AttThrust*4, ph_main, IspRCS, IspRCS);
	th_att_lin[3] = CreateThruster (_V( 19,0, 24.0), _V(0,0,-1), AttThrust*4, ph_main, IspRCS, IspRCS);
	CreateThrusterGroup (th_att_lin,   2, THGROUP_ATT_FORWARD);
	CreateThrusterGroup (th_att_lin+2, 2, THGROUP_ATT_BACK);
	AddExhaustStream(th_att_lin[1], _V( 35.0, 11.0,37.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[0], _V(-35.0, 11.0,37.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[1], _V( 35.0,-11.0,37.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[0], _V(-35.0,-11.0,37.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[3], _V( 35.0, 11.0,37.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[2], _V(-35.0, 11.0,37.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[3], _V( 35.0,-11.0,37.0),&RPL_exhaust);
	AddExhaustStream(th_att_lin[2], _V(-35.0,-11.0,37.0),&RPL_exhaust);
//	AddExhaust (th_att_lin[0], 1.5, 0.1, tex);
//	AddExhaust (th_att_lin[1], 1.5, 0.1, tex);
//	AddExhaust (th_att_lin[2], 1.5, 0.1, tex);
//	AddExhaust (th_att_lin[3], 1.5, 0.1, tex);
}



/* basic vessel class setup */

void ORION::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void ORION::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void ORION::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void ORION::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void ORION::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetOrionStage( );
}

void ORION::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetOrionStage( );
}

int ORION::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}

bool ORION::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{

	GetGlobalPos(GloPos);
	Global2Local(_V(-GloPos.x,-GloPos.y,-GloPos.z),LocPos);

	sunangle = atan(LocPos.x/LocPos.z);

	int d = hps->Markersize/2;
	int cx = hps->CX, cy = hps->CY;
	char pszBuffer[40];
	int bx = 0.5 * (cx + hps->W);

	sprintf(pszBuffer, "Power: %4.1f", (sqrt(fabs(sin(sunangle))))*cos(atan(LocPos.y/LocPos.x))*100.0);

	skp->Text(bx, 80.0, pszBuffer, strlen(pszBuffer));
		
	return true;
}


// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new ORION (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (ORION*)vessel;
}
