/**
*
* Tsiolkovsky v1.0
* (c) 2005 Erik Anderson (Sputnik)
* erik_h_anderson@hotmail.com
*
*
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
* 
*
* Requires CVEL library --                             
* please see ProjectGanymede on SourceForge for the latest source *
*/


#ifndef _TSIOLKOVSKY_H_
#define _TSIOLKOVSKY_H_


//#include <common_addon_ex.h>
#include <cvel.h>
#include <orbitersoundsdk3.h>
#include <cstdio>

using std::string;


	CONST double MainThrust =  501000;
	CONST double HoverThrust = 2e6;
	CONST double AttThrust = 10000;
	CONST double Ispmain = 10000;
	CONST double IspRCS = 5200;
	CONST double IspGYRO = 5200;
	CONST double SLAM_SPEED = 2.0;				// About 1/Hz, or a little slower
	CONST double PANELS_SPEED = 0.08;
	CONST double EmptyMass = 3000000.0;
	CONST double FuelMass = 12000000.0;
	CONST double FuelMassOberth = 24000000.0;
//	CONST double RCSFuelMass = 50000.0/2.205;
	CONST double DestructRange = 10000.0;
	CONST double FlashSize = 500.0;
	CONST VECTOR3 ExplosionPosition = _V(0,0,-50.0);


class ORION : public COMMON_VESSEL_EX {
public:
	ORION (OBJHANDLE hObj, int fmodel);
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount );
	void TimeStep(  double simt );
	void SlamCycle(double da);
	void VectorThrust();
	void PlaceComponents();
	void DestroyNeighbors();
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void InitialiseGuidance();
	void SetAttControlsOrion();
	void SetOrionStage();
	void SeparateEVA();
	void SetPanelsAnim();
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);
	void RefurbishOrion();	
	OBJHANDLE hV;
	int i, j;

	
// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);

protected:

	PROPELLANT_HANDLE ph_main, ph_RCS;
	THRUSTER_HANDLE th_main[1];
	THRUSTER_HANDLE th_orion[1];

	THGROUP_HANDLE thg_main;
	THGROUP_HANDLE thg_orion;
	MESHHANDLE  hTsiol, hTsiolFeet, hRock;
	SURFHANDLE  tex;
	DOCKHANDLE dock0, dock1, dock2, dock3, dock4, dock5, dock6;
	VECTOR3 AngularVel;
	char payloadmesh[256];
	char destructname[256];
	UINT anim_slam, anim_panels;
	UINT nuke;
	bool havesound;
	bool MoveNegative, firstnukestep;
	bool gpds_gui_active;
	bool UnDockArm;
	double distance;
	double slam_proc;
	double panels_proc;
//	double jiggle_proc;
	int numevas;
//	int bombcounter;
	double tempP, tempY;
	double da;
	VECTOR3 gorion, gpos, lpos;
	PSTREAM_HANDLE cloud;
	double DisplayTime, WaitTime, EngineDelay;
	int MyID;
	VECTOR3 GloPos,LocPos;
	double sunangle, solardist, sunmult, powermult;
	double fuelmass;
	bool Oberth;
	OBJHANDLE hvessel;

};

#endif
